#include "queue.h"
#include <stdlib.h>

int queue_init(queue_t *queue)
{
    queue->_head = NULL;
    queue->_tail = NULL;
    queue->_size = 0;
    return OK;
}


int queue_enqueue(queue_t *queue, void *data)
{
    if (data == NULL) {
        return NULL_ERR;
    }

    struct _node *to_add = (struct _node *) malloc(sizeof(struct _node));

    if (to_add == NULL) {
        return MALLOC_ERR;
    }

    to_add->_next_item = NULL;
    to_add->_data = data;

    if (queue->_size == 0) {
        queue->_head = queue->_tail = to_add;
    } else {
        queue->_tail->_next_item = to_add;
        queue->_tail = to_add;
    }

    queue->_size++;
    return OK;

}

void *queue_dequeue(queue_t *queue)
{
    if (queue->_size == 0) {
       return NULL;
    }

    struct _node *to_ret = queue->_head;
    queue->_head = to_ret->_next_item;

    if (queue->_head == NULL) {
        queue->_tail = NULL;
    }
    queue->_size--;

    void *result = to_ret->_data;
    free(to_ret);

    return result;
}

int queue_destroy(queue_t *queue, void (*deallocator)(void*))
{
    if (queue == NULL) {
        return NULL_ERR;
    }

    void *data;

    while ((data = queue_dequeue(queue)) != NULL) {
        if (deallocator != NULL) {
            deallocator(data);
        }
    }

    return OK;
}
